package com.ustadmobile.ustadapiconsumer.ui.screens

import android.app.Activity
import android.widget.Toast
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.ListItem
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.ClipboardManager
import androidx.compose.ui.platform.LocalClipboardManager
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.ustadmobile.ustadapiconsumer.GetOfflineAuthActivityResultContract
import com.ustadmobile.ustadapiconsumer.GetTokenResult
import com.ustadmobile.ustadapiconsumer.IpcClientBindActivity
import com.ustadmobile.ustadapiconsumer.util.ext.copyClickable
import com.ustadmobile.ustadapiconsumer.util.ext.getActivityContext

private fun Modifier.apiButton() = fillMaxWidth().padding(16.dp, 8.dp)

@Preview
@Composable
fun StartScreen(
    onClickAccountList: () -> Unit = {},
    tokenResult: GetTokenResult? = null,
    onTokenReceived: (GetTokenResult) -> Unit = {},
    onClickGetClassesForUser: () -> Unit = { },
    onClickPutLineItem: () -> Unit = { },
    onClickGetLineItem: () -> Unit = { },
    onClickPutResult: () -> Unit = { },
    onClickGetResultsForStudentForClass: () -> Unit = { },
) {
    val context = LocalContext.current

    val ipcActivity = (context.getActivityContext() as IpcClientBindActivity)

    val clientPort: Int by ipcActivity.httpPort.collectAsState(initial = 0)

    val launcher = rememberLauncherForActivityResult(
        contract = GetOfflineAuthActivityResultContract(),
        onResult = { result ->
            if(result.resultCode == Activity.RESULT_OK) {
                onTokenReceived(result)
                Toast.makeText(context, "Got token: ${result.accountName} / ${result.authToken}",
                    Toast.LENGTH_LONG).show()
            }else {
                Toast.makeText(context, "User declined", Toast.LENGTH_LONG).show()
            }
        }
    )

    val clipboardManager: ClipboardManager = LocalClipboardManager.current

    Column(
        modifier = Modifier.verticalScroll(rememberScrollState()),
    ) {
        Button(
            modifier = Modifier
                .padding(16.dp)
                .fillMaxWidth(),
            onClick = {
                launcher.launch(null)
            }
        ) {
            Text("Request Token")
        }

        tokenResult?.authToken?.also {
            ListItem(
                headlineContent = { Text(it) },
                supportingContent = { Text("Token", modifier = Modifier.testTag("token_label")) }
            )
        }

        tokenResult?.accountName?.also { accountName ->
            ListItem(
                modifier = Modifier
                    .copyClickable(accountName, clipboardManager, context),
                headlineContent = { Text(accountName) },
                supportingContent = { Text("Account Name") },
            )
        }

        tokenResult?.sourcedId?.also { sourcedId ->
            ListItem(
                modifier = Modifier
                    .copyClickable(sourcedId, clipboardManager, context),
                headlineContent = { Text(sourcedId) },
                supportingContent = { Text("Sourced Id") },
            )
        }

        tokenResult?.endpointUrl?.also { endpointUrl ->
            ListItem(
                modifier = Modifier
                    .copyClickable(endpointUrl, clipboardManager, context),
                headlineContent = { Text(endpointUrl) },
                supportingContent = { Text("Endpoint Url") },
            )
        }

        tokenResult?.onerosterUrl?.also { oneRosterUrl ->
            ListItem(
                modifier = Modifier
                    .copyClickable(oneRosterUrl, clipboardManager, context),
                headlineContent = { Text(oneRosterUrl) },
                supportingContent = { Text("OneRoster Url") },
            )
        }


        ListItem(
            modifier = Modifier
                .copyClickable("http://localhost:$clientPort/", clipboardManager, context),
            headlineContent = {
                Text(if(clientPort != 0) "http://localhost:$clientPort/" else "binding...")
            },
            supportingContent = { Text("HTTP Proxy") }
        )

        if(tokenResult != null) {
            Text("OneRoster APIs", modifier = Modifier.padding(16.dp, 8.dp))

            OutlinedButton(
                onClick = onClickGetClassesForUser,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(16.dp, 8.dp)
            ) {
                Text("Get Classes For User")
            }

            OutlinedButton(
                onClick = onClickPutLineItem,
                modifier = Modifier.apiButton(),
            ) {
                Text("Put Line Item")
            }

            OutlinedButton(onClick = onClickGetLineItem, modifier = Modifier.apiButton()) {
                Text("Get Line Item")
            }

            OutlinedButton(onClick = onClickPutResult, modifier = Modifier.apiButton()) {
                Text("Put Result")
            }

            OutlinedButton(
                onClick = onClickGetResultsForStudentForClass,
                modifier = Modifier.apiButton()
            ) {
                Text("Get Results For Student For Class")
            }
        }

    }
}