package com.ustadmobile.ustadapiconsumer.ui.screens.oneroster

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.ListItem
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalClipboardManager
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import com.ustadmobile.httpoveripc.core.SimpleTextRequest
import com.ustadmobile.httpoveripc.core.ext.asSimpleTextResponse
import com.ustadmobile.ustadapiconsumer.GetTokenResult
import com.ustadmobile.ustadapiconsumer.util.ext.copyClickable
import com.ustadmobile.ustadapiconsumer.util.ext.launchIpcRequest
import io.ktor.http.URLBuilder
import io.ktor.http.encodedPath
import io.ktor.http.takeFrom
import kotlinx.serialization.Serializable
import kotlinx.serialization.builtins.ListSerializer

@Serializable
data class ClazzData(
    val sourcedId: String,
    val title: String,
)


@Composable
fun GetClassesForUserScreen(
    tokenResult: GetTokenResult?
) {
    val scope = rememberCoroutineScope()
    val context = LocalContext.current

    var clazzList by remember {
        mutableStateOf(emptyList<ClazzData>())
    }

    var rawJson: String? by remember {
        mutableStateOf(null)
    }

    LaunchedEffect(Unit) {
        scope.launchIpcRequest(context) { ipcContext ->
            val getClassesUrl = URLBuilder().takeFrom(
                tokenResult?.onerosterUrl ?: ""
            ).apply {
                encodedPath = "${encodedPath}/users/${tokenResult?.sourcedId}/classes"
            }.build()

            val response = ipcContext.ipcClient.send(
                SimpleTextRequest(
                    method = SimpleTextRequest.Method.GET,
                    url = getClassesUrl,
                    headers = tokenResult?.authHeaders() ?: emptyMap()
                ).toRawHttpRequest(ipcContext.rawHttp)).asSimpleTextResponse()

            val clazzes = ipcContext.json.decodeFromString(
                ListSerializer(ClazzData.serializer()), response.responseBody ?: "[]"
            )

            clazzList = clazzes

            rawJson = response.responseBody
        }
    }


    LazyColumn {
        item {
            Text(modifier = Modifier.padding(16.dp, 8.dp), text = "Classes:")
            HorizontalDivider()
        }

        items(clazzList) { clazz ->
            ListItem(
                headlineContent = { Text(clazz.title) },
                supportingContent = {
                    Text(
                        text = clazz.sourcedId,
                        modifier = Modifier.copyClickable(
                            clazz.sourcedId, LocalClipboardManager.current, context
                        )
                    )
                }
            )
        }

        item {
            Column {
                Text(modifier = Modifier.padding(16.dp, 8.dp), text = "JSON response")
                HorizontalDivider()
                Text(
                    modifier = Modifier.padding(16.dp, 8.dp),
                    text = rawJson ?: "",
                )
            }

        }
    }

}