package com.ustadmobile.ustadapiconsumer.ui.screens.oneroster

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import com.ustadmobile.httpoveripc.core.SimpleTextRequest
import com.ustadmobile.httpoveripc.core.ext.asSimpleTextResponse
import com.ustadmobile.ustadapiconsumer.GetTokenResult
import com.ustadmobile.ustadapiconsumer.util.ext.launchIpcRequest
import io.ktor.http.URLBuilder
import io.ktor.http.encodedPath
import io.ktor.http.takeFrom
import java.net.URLEncoder

@Composable
fun GetLineItemScreen(
    tokenResult: GetTokenResult?
) {
    var sourcedId by remember {
        mutableStateOf("")
    }

    val scope = rememberCoroutineScope()
    val context = LocalContext.current

    var statusCode: Int by remember {
        mutableIntStateOf(0)
    }

    var responseJson: String? by remember {
        mutableStateOf(null)
    }

    Column(
        modifier = Modifier.verticalScroll(rememberScrollState())
    ) {
        OutlinedTextField(
            value = sourcedId,
            onValueChange = { sourcedId = it },
            label = { Text("LineItem SourcedId") },
            modifier = Modifier
                .fillMaxWidth()
                .padding(start = 16.dp, end = 16.dp, top = 16.dp, bottom = 8.dp)
        )

        Button(
            onClick = {
                scope.launchIpcRequest(context) { ipcContext ->
                    val url = URLBuilder().takeFrom(
                        tokenResult?.onerosterUrl ?: ""
                    ).apply {
                        encodedPath = "${encodedPath}/lineItems/${URLEncoder.encode(sourcedId, "UTF-8")}"
                    }.build()

                    val response = ipcContext.ipcClient.send(
                        SimpleTextRequest(
                            method = SimpleTextRequest.Method.GET,
                            url = url,
                            headers = tokenResult?.authHeaders() ?: emptyMap()
                        ).toRawHttpRequest(ipcContext.rawHttp)
                    ).asSimpleTextResponse()
                    statusCode = response.statusCode
                    responseJson = response.responseBody
                }
            },
            modifier = Modifier.fillMaxWidth().padding(16.dp, 8.dp)
        ) {
            Text("GET")
        }

        if(statusCode != 0) {
            Text(statusCode.toString(), modifier = Modifier.padding(16.dp, 8.dp))
            Text(responseJson ?: "", modifier = Modifier.padding(16.dp, 8.dp))
        }
    }
}
