package com.ustadmobile.ustadapiconsumer.ui.screens.oneroster

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Button
import androidx.compose.material3.ListItem
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import com.ustadmobile.httpoveripc.core.SimpleTextRequest
import com.ustadmobile.httpoveripc.core.ext.asSimpleTextResponse
import com.ustadmobile.ustadapiconsumer.GetTokenResult
import com.ustadmobile.ustadapiconsumer.util.ext.launchIpcRequest
import io.ktor.http.URLBuilder
import io.ktor.http.encodedPath
import io.ktor.http.takeFrom
import kotlinx.serialization.Serializable
import kotlinx.serialization.builtins.ListSerializer

@Serializable
data class GUIDRef(
    val href: String,
    val sourcedId: String,
)

@Serializable
data class OneRosterResult(
    val sourcedId: String,
    val lineItem: GUIDRef,
    val student: GUIDRef,
    val score: Float,
)

@Composable
fun GetResultsForStudentForClassScreen(
    tokenResult: GetTokenResult?
) {
    var studentSourcedId: String by remember { mutableStateOf(tokenResult?.sourcedId ?: "") }
    var classSourcedId: String by remember { mutableStateOf("") }

    var results: List<OneRosterResult> by remember { mutableStateOf(emptyList()) }

    var rawJson: String? by remember { mutableStateOf(null) }

    val scope = rememberCoroutineScope()
    val context = LocalContext.current


    LazyColumn {
        item {
            OutlinedTextField(
                value = studentSourcedId,
                onValueChange = { studentSourcedId = it },
                label = { Text("Student SourcedId") },
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 16.dp, start = 16.dp, end = 16.dp, bottom = 8.dp),
            )
        }

        item {
            OutlinedTextField(
                value = classSourcedId,
                onValueChange = { classSourcedId = it },
                label = { Text("Class SourcedId") },
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(16.dp, 8.dp)
            )
        }

        item {
            Button(
                onClick = {
                    scope.launchIpcRequest(context) { ipcContext ->
                        val resultsForStudentForClass = URLBuilder().takeFrom(
                            tokenResult?.onerosterUrl ?: ""
                        ).apply {
                            encodedPath = "${encodedPath}/classes/$classSourcedId/students/$studentSourcedId/results"
                        }.build()

                        val response = ipcContext.ipcClient.send(
                            SimpleTextRequest(
                                method = SimpleTextRequest.Method.GET,
                                url = resultsForStudentForClass,
                                headers = tokenResult?.authHeaders() ?: emptyMap()
                            ).toRawHttpRequest(ipcContext.rawHttp)).asSimpleTextResponse()

                        val resultsList = ipcContext.json.decodeFromString(
                            ListSerializer(OneRosterResult.serializer()), response.responseBody ?: "[]"
                        )
                        results = resultsList

                        rawJson = response.responseBody
                    }
                },
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(16.dp, 8.dp)
            ) {
                Text("GET")
            }
        }

        items(results) { result ->
            ListItem(
                headlineContent = { Text(result.sourcedId) },
                supportingContent = {
                    Column {
                        Text("Line Item: ${result.lineItem.sourcedId}")
                        Text("Student: ${result.student.sourcedId}")
                        Text("Score: ${result.score}")
                    }
                }
            )
        }

        rawJson?.also {
            item {
                Text("Raw Json", modifier = Modifier.padding(16.dp, 8.dp))
            }

            item {
                Text(it, modifier = Modifier.padding(16.dp, 8.dp))
            }
        }

    }
}