package com.ustadmobile.ustadapiconsumer.ui.screens.oneroster

import androidx.compose.runtime.Composable
import com.ustadmobile.httpoveripc.core.SimpleTextRequest
import com.ustadmobile.httpoveripc.core.ext.asSimpleTextResponse
import com.ustadmobile.ustadapiconsumer.GetTokenResult
import io.ktor.http.URLBuilder
import io.ktor.http.encodedPath
import io.ktor.http.takeFrom
import kotlinx.serialization.json.JsonObject
import kotlinx.serialization.json.jsonPrimitive
import java.net.URLEncoder

@Composable
fun PutLineItemScreen(
    tokenResult: GetTokenResult?
) {
    PutScreen(
        textFieldLabel = "LineItem (JSON)",
        defaultJson = """
            {
                  "sourcedId": "line-item-sourced-id",
                  "status": "active",
                  "dateLastModified": "2023-03-14T10:09:04.004Z",
                  "metadata": {},
                  "title": "Your-Title",
                  "description": "Your-description",
                  "assignDate": "2023-03-14T10:09:04.004Z",
                  "dueDate": "2024-03-14T10:09:04.013Z",
                  "class": {
                    "href": "clazz-sourced-id",
                    "sourcedId": "clazz-sourced-id",
                    "type": "class"
                  },
                  "category": {
                    "href": "category",
                    "sourcedId": "category",
                    "type": "category"
                  },
                  "resultValueMin": 0,
                  "resultValueMax": 100
                }
        """.trimIndent(),
        onSendPut = { ipcContext, lineItemJson ->
            val lineItemObj: JsonObject = ipcContext.json.decodeFromString(lineItemJson)
            val lineItemId = lineItemObj["sourcedId"]?.jsonPrimitive?.content
                ?: throw IllegalArgumentException("No sourcedid")

            val putLineItemUrl = URLBuilder().takeFrom(
                tokenResult?.onerosterUrl ?: ""
            ).apply {
                encodedPath = "${encodedPath}/lineItems/${URLEncoder.encode(lineItemId, "UTF-8")}"
            }.build()

            ipcContext.ipcClient.send(
                SimpleTextRequest(
                    method = SimpleTextRequest.Method.PUT,
                    url = putLineItemUrl,
                    headers = tokenResult?.authHeaders() ?: emptyMap(),
                    requestBody = lineItemJson,
                ).toRawHttpRequest(ipcContext.rawHttp)
            ).asSimpleTextResponse()
        }
    )
}