package com.ustadmobile.ustadapiconsumer.ui.screens.oneroster

import androidx.compose.runtime.Composable
import com.ustadmobile.httpoveripc.core.SimpleTextRequest
import com.ustadmobile.httpoveripc.core.ext.asSimpleTextResponse
import com.ustadmobile.ustadapiconsumer.GetTokenResult
import io.ktor.http.URLBuilder
import io.ktor.http.encodedPath
import io.ktor.http.takeFrom
import kotlinx.serialization.json.JsonObject
import kotlinx.serialization.json.jsonPrimitive
import java.net.URLEncoder

@Composable
fun PutResultScreen(
    tokenResult: GetTokenResult?
) {
    PutScreen(
        textFieldLabel ="Result (JSON)",
        defaultJson = """
            {
              "sourcedId": "result-sourced-id",
              "status": "active",
              "dateLastModified": "2024-05-12T16:47:14.044Z",
              "metaData": null,
              "lineItem": {
                "href": "http://localhost",
                "sourcedId": "lineitem-sourced-id",
                "type": "lineItem"
              },
              "student": {
                "href": "http://localhost/",
                "sourcedId": "student-sourced-id",
                "type": "student"
              },
              "score": 5,
              "scoreDate": "2024-05-12T16:47:14.046Z",
              "comment": "OK, not great, not terrible"
            }
        """.trimIndent(),
        onSendPut = { ipcContext, json ->
            val resultObj: JsonObject = ipcContext.json.decodeFromString(json)
            val sourcedId = resultObj["sourcedId"]?.jsonPrimitive?.content
                ?: throw IllegalArgumentException("No sourcedid")

            val putResultUrl = URLBuilder().takeFrom(
                tokenResult?.onerosterUrl ?: ""
            ).apply {
                encodedPath = "${encodedPath}/results/${URLEncoder.encode(sourcedId, "UTF-8")}"
            }.build()

            ipcContext.ipcClient.send(
                SimpleTextRequest(
                    method = SimpleTextRequest.Method.PUT,
                    url = putResultUrl,
                    headers = tokenResult?.authHeaders() ?: emptyMap(),
                    requestBody = json,
                ).toRawHttpRequest(ipcContext.rawHttp)
            ).asSimpleTextResponse()
        }
    )
}