package com.ustadmobile.ustadapiconsumer.ui.screens.oneroster

import android.util.Log
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.ListItem
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import com.ustadmobile.httpoveripc.core.SimpleTextResponse
import com.ustadmobile.ustadapiconsumer.util.ext.IpcRequestContext
import com.ustadmobile.ustadapiconsumer.util.ext.launchIpcRequest

@Composable
fun PutScreen(
    textFieldLabel: String,
    defaultJson: String,
    onSendPut: suspend (IpcRequestContext, String) -> SimpleTextResponse,
) {
    val context = LocalContext.current
    val scope = rememberCoroutineScope()

    var putBodyJson: String by remember {
        mutableStateOf(defaultJson)
    }

    var statusCode by remember {
        mutableIntStateOf(0)
    }

    var responseBody: String? by remember {
        mutableStateOf(null)
    }

    var errMessage: String? by remember {
        mutableStateOf(null)
    }


    Column(
        modifier = Modifier.verticalScroll(rememberScrollState())
    ) {

        OutlinedTextField(
            value = putBodyJson,
            modifier = Modifier
                .fillMaxWidth()
                .padding(horizontal = 16.dp, vertical = 8.dp),
            label = { Text(textFieldLabel) },
            maxLines = 20,
            onValueChange = {
                putBodyJson = it
            }
        )

        Button(
            modifier = Modifier
                .fillMaxWidth()
                .padding(horizontal = 16.dp, vertical = 8.dp),
            onClick = {
                scope.launchIpcRequest(context) { ipcContext ->
                    try {
                        statusCode = 0
                        responseBody = null

                        val response = onSendPut(ipcContext, putBodyJson)

                        statusCode = response.statusCode
                        responseBody = response.responseBody
                    }catch(e: Throwable) {
                        errMessage = e.message
                        Log.e("PutScreen", e.message.toString(), e)
                    }
                }
            }
        ) {
            Text("PUT")
        }

        if(statusCode != 0) {
            ListItem(
                headlineContent = {
                    Text("$statusCode ${responseBody ?: ""}")
                },
                supportingContent = {
                    Text("Response")
                }
            )
        }

        errMessage?.also { error ->
            Text(modifier = Modifier.padding(16.dp, 8.dp), text = error)
        }
    }

}